# -*- coding: utf-8 -*-

"""Utilities for acquiring and formatting triples."""

from .data_transformation_utils import transform_freebase_to_internal_format, transform_wn_18_to_internal_format
from .instance_creation_utils import create_mapped_triples, create_mappings
